﻿using UnityEngine;
using System.Collections;

public class CustomBindings : MonoBehaviour {
	public int Index;
	public bool IsAxis = false;
	public string MappingName;
	public bool PositiveAxis = false;
    [System.Serializable]
    public class TextEvent : UnityEngine.Events.UnityEvent<string> { }
    public TextEvent OnTextUpdate;
	//public UILabel DisplayLabel;
	void Awake() {
		if(IsAxis) {
			RInput.OnAxisUpdated += HandleOnAxisUpdated;
		} else {
			RInput.OnMappingUpdated += HandleOnMappingUpdated;
		}
		UpdateLabel();
	}
	void OnDestroy() {
		RInput.OnAxisUpdated -= HandleOnAxisUpdated;
		RInput.OnMappingUpdated -= HandleOnMappingUpdated;
		
	}	
	void UpdateLabel() {
		KeyCode currentCode = KeyCode.None;
		if(IsAxis) {			
			currentCode = RInput.GetAxisLabel(MappingName,PositiveAxis,Index);
		} else {
			currentCode= RInput.GetMappingLabel(MappingName,Index);
		}
        OnTextUpdate.Invoke( ConvertCodeToEnglish(currentCode.ToString())); 
	}
	public string ConvertCodeToEnglish(string code) {
		if(code.ToLower().StartsWith("alpha")) {
			return code.Substring(5);
		}
		if(code.ToLower().StartsWith("keypad")) {
			return "Keypad " + code.Substring(6);
		}
		if(code.ToLower().EndsWith("arrow")) {
			return code.Substring(0,code.ToLower().IndexOf("arrow")) + " Arrow";
		}
		
		if(code.ToLower().EndsWith("apple")) {
			string OS = "OS";
			switch(Application.platform) {
			case RuntimePlatform.WindowsPlayer:
			case RuntimePlatform.WindowsEditor:
			case  RuntimePlatform.WindowsWebPlayer:
				OS = "Windows";
				
				break;
			case RuntimePlatform.OSXEditor:
			case RuntimePlatform.OSXDashboardPlayer:
			case  RuntimePlatform.OSXPlayer:
			case RuntimePlatform.OSXWebPlayer:
				OS = "Apple";
				
				break;
			}
			return code.Substring(0,code.ToLower().IndexOf("apple")) + " " + OS + " Key";
		}
		return code;
	}

	public void ChangeThisBinding() {
		RInput.Instance.BeginListenForKeyPress(this.MappingName,this.Index,IsAxis,PositiveAxis);
	}
	void HandleOnMappingUpdated (Object sender, string e)
	{
		UpdateLabel();
	}

	void HandleOnAxisUpdated (Object sender, string e)
	{
		UpdateLabel();
		
	}
}
