﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
public class RInput : MonoBehaviour {
	public Transform Popup;
	public delegate void EventHandler<TEventArgs>(Object sender, TEventArgs e);
	public static event EventHandler<string> OnMappingUpdated;
	public static event EventHandler<string> OnAxisUpdated;
	public List<InputMap> DefaultMaps;
	public List<InputAxis> DefaultAxes;
	
	public List<InputMap> _InputMaps;
	public List<InputAxis> _InputAxes;
	public static List<InputMap> InputMaps { get { return Instance._InputMaps; }}
	public static List<InputAxis> InputAxes{ get { return Instance._InputAxes; }}
	
	public KeyCode[] ValidKeyCodes;
	
	void Awake() {
		if(RInput.Instance != this) {
			DestroyImmediate(this.gameObject);
		}
        canRebind = true;
		ValidKeyCodes = (KeyCode[])System.Enum.GetValues(typeof(KeyCode));
		LoadFromSettings();
		if(Popup != null) Popup.gameObject.SetActive(false);
	}
    void OnEnable()
    {
        canRebind = true;
    }
	[ContextMenu("Set Defaults")]
	private void SetDefaults() {
		DefaultMaps = _InputMaps.ToArray().ToList();
		DefaultAxes = _InputAxes.ToArray().ToList();
	} 
	private void GetAxisSaved(InputAxis Existing, int index) {  
		Existing.Name = PlayerPrefs.GetString(string.Format("{0}_AxisName",index),Existing.Name); 
		Existing.Positive = (KeyCode)PlayerPrefs.GetInt(string.Format("{0}_PositiveAxis",index),(int)Existing.Positive);
		Existing.Negative = (KeyCode)PlayerPrefs.GetInt(string.Format("{0}_NegativeAxis",index),(int)Existing.Negative);
		Existing.UnityAxis = PlayerPrefs.GetString(string.Format("{0}_UnityAxis",index),Existing.UnityAxis); 
	}
	private void GetMapSaved(InputMap Existing, int index) {  
		Existing.Name = PlayerPrefs.GetString(string.Format("{0}_MapName",index),Existing.Name); 
		Existing.keyCode = (KeyCode)PlayerPrefs.GetInt(string.Format("{0}_mapKeyCode",index),(int)Existing.keyCode); 
		Existing.UnityAxis = PlayerPrefs.GetString(string.Format("{0}_mapUnityButton",index),Existing.UnityAxis); 
	}
	private void SetAxisSaved(InputAxis Existing, int index) {   
		PlayerPrefs.SetString(string.Format("{0}_AxisName",index),Existing.Name);
		PlayerPrefs.SetInt(string.Format("{0}_PositiveAxis",index),(int)Existing.Positive);
		PlayerPrefs.SetInt(string.Format("{0}_NegativeAxis",index),(int)Existing.Negative);
		PlayerPrefs.SetString(string.Format("{0}_UnityAxis",index),Existing.UnityAxis); 
	}
	private void SetMapSaved(InputMap Existing, int index) {  
		PlayerPrefs.SetString(string.Format("{0}_MapName",index),Existing.Name);
		PlayerPrefs.SetInt( string.Format("{0}_mapKeyCode",index),(int)Existing.keyCode); 
		PlayerPrefs.SetString( string.Format("{0}_mapUnityButton",index),Existing.UnityAxis); 
	}
    private bool canRebind = true;
    private IEnumerator KeybindingCooldown()
    {
        canRebind = false;
        yield return new WaitForSeconds(0.2f);
        Debug.Log("Keybinding Cooldown finished");
        canRebind = true;
    }
	public void BeginListenForKeyPress(string Mapping, int index,bool isAxis, bool Positive) {
        if (!canRebind) return;
        canRebind = false;
		StopCoroutine("ListenForKey");
		StartCoroutine(ListeningForKey(Mapping, index, isAxis,  Positive));
	}
    public void StopListening()
    {
        StopCoroutine("ListeningForKey");
        StartCoroutine(KeybindingCooldown());
        if (Popup != null) Popup.gameObject.SetActive(false);
    }
	IEnumerator ListeningForKey(string Mapping, int index,bool isAxis, bool Positive) {
		if(Popup != null) Popup.gameObject.SetActive(true);
		yield return null;
		Debug.Log("Waiting for Key Press, " + Mapping + "," + index+ ", " + isAxis+ ", " + Positive);
		bool running = true;
		while(running) {
			yield return null;
            if (GetButton(RInputEventsSystem.Instance.cancelButton))
            {
                running = false;
            } else 
			if(Input.anyKeyDown) {
			
				KeyCode code = KeyCode.None;
				for(int i = 0; i < ValidKeyCodes.Length; i++) {
					if(Input.GetKeyDown(ValidKeyCodes[i])) {
						code = ValidKeyCodes[i];
						break;
					}
				}
				Debug.Log("CODE PRESSED:" + code);
				if(isAxis) {
					SetAxis(Mapping,Positive,code,index);	
				} else {
					SetMapping(Mapping,code,index);
				} 
				running = false;
			}
        }
        StartCoroutine(KeybindingCooldown());
		if(Popup != null) Popup.gameObject.SetActive(false);
	}
	private void SetMapping(string Mapping, KeyCode Key, int index) {
		InputMap Existing = RInput.InputMaps.Where(x=>x.Name == Mapping).ToArray()[index];
		if(Existing == null) return;
		Existing.keyCode = Key;
		OnMappingUpdated.Invoke(this,Mapping);
	}
	private void SetAxis(string AxisName, bool Positive, KeyCode Key, int index) {
		
		InputAxis Existing = RInput.InputAxes.Where(x=>x.Name == AxisName).ToArray()[index];
		if(Existing == null) return;
		if(Positive) {
			Existing.Positive = Key;
		} else {
			Existing.Negative = Key;
		}
		OnAxisUpdated.Invoke(this,AxisName);
	} 
	public void RestoreDefaults() {
		_InputAxes = DefaultAxes.Select((x)=> {
			return new InputAxis() {
				UnityAxis = x.UnityAxis,
				Name = x.Name,
				Positive = x.Positive,
				Negative = x.Negative				
			};
		}).ToList();
		_InputMaps = DefaultMaps.Select((x)=> {
			return new InputMap() {
				UnityAxis = x.UnityAxis,
				Name = x.Name,
				keyCode = x.keyCode
			};
		}).ToList();
		OnMappingUpdated.Invoke(this,"All");	
		OnAxisUpdated.Invoke(this,"All");
	}
    [ContextMenu("Transfer current to defaults")]
    public void TransferCurrentToDefault()
    {
        this.DefaultAxes = this._InputAxes.Select(x => new InputAxis()
        {
            Name = x.Name,
            Negative = x.Negative,
            Positive = x.Positive,
            UnityAxis = x.UnityAxis
        }).ToList();
        this.DefaultMaps = this._InputMaps.Select(x => new InputMap()
        {
            Name = x.Name,
            keyCode = x.keyCode,
            UnityAxis = x.UnityAxis
        }).ToList();
    }
	[ContextMenu("Load from Player Prefs")]
	public void LoadFromSettings() {
		for(int i = 0;i < RInput.InputAxes.Count(); i++) {
			GetAxisSaved(RInput.InputAxes[i],i);
		}		
		for(int i = 0;i < RInput.InputMaps.Count(); i++) {
			GetMapSaved(RInput.InputMaps[i],i);
		}	
		if(OnMappingUpdated != null) {
		OnMappingUpdated.Invoke(this,"All");	
		}
		if(OnAxisUpdated != null) {
		OnAxisUpdated.Invoke(this,"All");	
		}
	}
	[ContextMenu("Save to Player Prefs")]
	public void SaveToSettings() {
		
		for(int i = 0;i < RInput.InputAxes.Count(); i++) {
			SetAxisSaved(RInput.InputAxes[i],i);
		}		
		for(int i = 0;i < RInput.InputMaps.Count(); i++) {
			SetMapSaved(RInput.InputMaps[i],i);
		}	
        if(OnMappingUpdated != null)
            OnMappingUpdated.Invoke(this, "All");
        if (OnAxisUpdated != null)	
		OnAxisUpdated.Invoke(this,"All");
	}
	private static RInput _instance;
	public static RInput Instance {get {
		if(_instance == null) {
			_instance = GameObject.FindObjectOfType(typeof(RInput)) as RInput;
		}
		return _instance;
	}
	}
	[System.Serializable]
	public class InputAxis {
		public string Name;
		public KeyCode Positive,Negative;
		public string UnityAxis;
		public float Value {
			get {
				if(!string.IsNullOrEmpty(UnityAxis)) {
					var v = Input.GetAxis(UnityAxis);
					if(v != 0) return v;
				} 
				if(Input.GetKey(Positive)) return 1;
				if(Input.GetKey(Negative)) return -1;
				return 0;
			}
		}
	}
	[System.Serializable]
	public class InputMap {
		public string Name;
		public KeyCode keyCode;	
		public string UnityAxis;
		public bool IsPressed {
			get {
				bool isPressed = false;
				if(keyCode != KeyCode.None) {
					isPressed |= Input.GetKey(keyCode);
				}
				if(!string.IsNullOrEmpty(UnityAxis)) {
					isPressed |=Input.GetButton(UnityAxis);
				}
				return isPressed;
			}
		}
		public bool IsDown {
			get {
				bool isPressed = false;
				if(keyCode != KeyCode.None) {
					isPressed |= Input.GetKeyDown(keyCode);
				}
				if(!string.IsNullOrEmpty(UnityAxis)) {
					isPressed |=Input.GetButtonDown(UnityAxis);
				}
				return isPressed;		 		
			}
		}
		public bool IsReleased {
			get {
				bool isPressed = false;
				if(keyCode != KeyCode.None) {
					isPressed |= Input.GetKeyUp(keyCode);
				}
				if(!string.IsNullOrEmpty(UnityAxis)) {
					isPressed |=Input.GetButtonUp(UnityAxis);
				}
				return isPressed;		 			
			}
		}
	} 
	public static KeyCode GetAxisLabel(string Mapping, bool Positive, int Index) {
		var axis = InputAxes.Where(x=>x.Name == Mapping).ToArray()[Index];
		if(Positive) return axis.Positive;
		return axis.Negative;
		
	}
	public static KeyCode GetMappingLabel(string Mapping, int Index) {
		
		var axis = InputMaps.Where(x=>x.Name == Mapping).ToArray()[Index];
		return axis.keyCode;
		
	}
	public static float GetAxis(string Mapping) {
		float value = 0;
		foreach(var mapping in InputAxes) {
			if(mapping.Name.ToLower() == Mapping.ToLower()) {
				value = mapping.Value;
				if(value != 0) return value;				
			}		
		}
		return 0;
	}
	public static bool GetButton(string Mapping) {
		bool IsPressed = false;
		foreach(var mapping in InputMaps) {
			if(mapping.Name.ToLower() == Mapping.ToLower()) {
				IsPressed = IsPressed || mapping.IsPressed;
			}		
		}
		return IsPressed;
	}
	public static bool GetButtonDown(string Mapping) {
		bool isDown = false;
		foreach(var mapping in InputMaps) {
			if(mapping.Name.ToLower() == Mapping.ToLower()) {
				isDown = isDown || mapping.IsDown;
			}		
		}
		return isDown;
	}
	public static bool GetButtonUp(string Mapping) {
		bool IsReleased = false;
		foreach(var mapping in InputMaps) {
			if(mapping.Name.ToLower() == Mapping.ToLower()) {
				IsReleased = IsReleased || mapping.IsReleased;
			}		
		}
		return IsReleased;
	}
	
}
