﻿using UnityEngine;
using System.Collections;
using UnityEngine.EventSystems; 
public class RInputEventsSystem : StandaloneInputModule
{
    public static RInputEventsSystem Instance
    {
        get
        {
            return GameObject.FindObjectOfType<RInputEventsSystem>();
        }
    }
    public float lastInputTime = 0;
    protected override AxisEventData GetAxisEventData(float x, float y, float moveDeadZone)
    { 
        var horizontalAxis = RInput.GetAxis(this.horizontalAxis);
        var verticalAxis = RInput.GetAxis(this.verticalAxis);
        return base.GetAxisEventData(horizontalAxis, verticalAxis, moveDeadZone); 
    }
    public override bool ShouldActivateModule()
    {
        if (!base.ShouldActivateModule())
            return false;

        var shouldActivate = RInput.GetButton(this.submitButton);
        shouldActivate |= RInput.GetButton(this.cancelButton);
        shouldActivate |= !Mathf.Approximately(RInput.GetAxis(this.horizontalAxis), 0.0f);
        shouldActivate |= !Mathf.Approximately(RInput.GetAxis(this.verticalAxis), 0.0f);
        return shouldActivate;
    }
    public override void Process()
    {
        base.Process();
        SendSubmitEventToSelectedObject();
         
    }
    private bool SendSubmitEventToSelectedObject()
    {
        if (eventSystem.currentSelectedGameObject == null)
            return false;
        if (lastInputTime > Time.time - 0.5f) return false;
        var data = GetBaseEventData();

        if (RInput.GetButton(this.submitButton))
            ExecuteEvents.Execute(eventSystem.currentSelectedGameObject, data, ExecuteEvents.submitHandler);

        if( RInput.GetButton(this.cancelButton))
            ExecuteEvents.Execute(eventSystem.currentSelectedGameObject, data, ExecuteEvents.cancelHandler);
        return data.used;
    }


    public BaseEventData OverrideGetBaseEventData()
    {
        return base.GetBaseEventData();
    }
}
