﻿using UnityEngine;
using System.Collections;
using UnityEngine.Events;
using System.Collections.Generic; 
public class Follow : MonoBehaviour {
    public Transform Target;
    public Camera cam;
    public AudioClip[] Swish;
    public float Aspect = 4f / 3f;
    private int _swishn = 0;
    [System.Serializable]
    public class ScreenShow
    {
        public Transform Screen;
        public int X, Y;
    }
    public List<ScreenShow> ScreenShowItems;
    AudioClip _swish
    {
        get
        {
            return Swish[(++_swishn) % Swish.Length];
        }
    }
    public float Speed = 1;
    public float Step = 1;
    public UnityEvent OnScreenChange;
    public float ortho
    {
        get
        {
            return cam.orthographicSize;
        }
    }
    void Awake()
    {
        cam.aspect = Aspect;

        var TargetPosition = GetTargetPosition(Target.position.x, Target.position.y);
        GridX = GetGridCoords(TargetPosition.x, Step * ortho * Aspect);
        GridY = GetGridCoords(TargetPosition.y, Step * ortho);
        SetupGridOff(GridX, GridY);
    }
    void OnDrawGizmos()
    {
        Gizmos.color = Color.red;
        for (float ix = 0; ix < 100; ix += (Step * ortho * Aspect))
        {

            for (float iy = 0; iy < 40; iy += (Step * ortho))
            {
                Gizmos.DrawWireCube(GetTargetPosition(ix, iy), new Vector3(Step * ortho * Aspect, Step * ortho, 0));
            }
        }
    }
    Vector3 GetTargetPosition(float x, float y)
    {
            return new Vector3(
                SnapTo(x, Step * ortho * Aspect),
                SnapTo(y, Step * ortho),
                this.transform.position.z);
     
    }
    float SnapTo(float t, float s)
    {
        return Mathf.RoundToInt(t / s) * s;
    }
    int GetGridCoords(float t, float s)
    {
        return Mathf.RoundToInt(t / s);
    } 
	// Update is called once per frame
    public int GridX, GridY;
	void Update () { 
        var TargetPosition = GetTargetPosition(Target.position.x, Target.position.y);
        var distance = Vector3.Distance(this.transform.position, TargetPosition);
        if (distance > 0)
        {
            if (Time.timeScale != 0)
            {
                if (OnScreenChange != null) OnScreenChange.Invoke();
                GridX = GetGridCoords(TargetPosition.x, Step * ortho * Aspect);
                GridY = GetGridCoords(TargetPosition.y, Step * ortho);
                SetupGridOn(GridX, GridY);
                AudioSource.PlayClipAtPoint(_swish, this.transform.position);
            }
            Time.timeScale = 0; 
            this.transform.position = Vector3.Lerp(this.transform.position, TargetPosition, Speed / distance);
        }
        else if(Time.timeScale == 0)
        {
            GridX = GetGridCoords(TargetPosition.x, Step * ortho * Aspect);
            GridY = GetGridCoords(TargetPosition.y, Step * ortho);
            SetupGridOff(GridX, GridY);
            Time.timeScale = 1;
        }
	}
    void SetupGridOff(int x, int y)
    {
        if(Target.tag =="Player")        Target.parent = null;
        foreach (var item in this.ScreenShowItems)
        {
            if (item.X != x || item.Y != y)
            { 
                item.Screen.gameObject.SetActive(false);
            }
        }
    }
    void SetupGridOn(int x, int y)
    {
        if (Target.tag == "Player") Target.parent = null;
        foreach (var item in this.ScreenShowItems)
        {
            if (item.X == x && item.Y == y)
            {
                item.Screen.gameObject.SetActive(true);
                 
            }
        }
    }
}
