﻿using UnityEngine;
using System.Collections;

public class WallJump : MonoBehaviour {
    public Simple2dPlatformController Controller;
    public LayerMask WallJumpableSurfaces;
    public Vector2 WallJumpForce;
    public float WallJumpTestDistance = 0.2f;
    bool JumpEnabled = true;
    public bool RepeatableJump = false;
    public void EnableRepeatableJump()
    {
        RepeatableJump = true;
    }
    public void EnableSuperWallJump()
    {
        WallJumpForce.y = 10;
    }
    void Awake()
    {
        if (Controller == null)
        {
            Controller = this.GetComponent<Simple2dPlatformController>();
        }
    }
    private bool isGrounded = true;

    void Update()
    {
        if (isGrounded && !Controller.isGrounded)
        {
            JumpEnabled = false;
            isGrounded = false;
            StartCoroutine(checkJumpButton());
        }
        if (Controller.isGrounded)
        {
            isGrounded = true;

        }
    }
    private IEnumerator checkJumpButton() {
        yield return new WaitForSeconds(0.1f);
        while (true)
        {
            if (!Controller.isGrounded)
            {
                if (RInput.GetButton("Jump") && JumpEnabled)
                {
                    JumpEnabled = false;
                    StartCoroutine(CheckWallJump());
                    if (!RepeatableJump)
                    {
                        break;
                    }
                    else
                    {
                        yield return new WaitForSeconds(0.1f);
                    }
                }
                if (!RInput.GetButton("Jump") && !JumpEnabled)
                {
                    JumpEnabled = true;
                }
            }
            yield return null;
            if (isGrounded) break;
        }

	}
    public IEnumerator CheckWallJump()
    {
        for (float i = 0; i < 0.1f; i += Time.deltaTime)
        {
            yield return new WaitForEndOfFrame();
            if (JumpEnabled) break;
            if (WallTest())
            {
                break;
            }
        }
    }
    /// <summary>
    /// Used in Raycast shortcut
    /// </summary>
    RaycastHit2D lastHitResult = new RaycastHit2D();
    /// <summary>
    /// Raycast shortcut
    /// </summary>
    /// <param name="Direction"></param>
    /// <param name="mask"></param>
    /// <returns>True on hit, False on not hit</returns>
    public bool Raycast(Vector2 Direction, LayerMask mask)
    {
        lastHitResult = Physics2D.Raycast(this.transform.position, Direction, WallJumpTestDistance, mask);
        if (lastHitResult != null && lastHitResult.collider != null) return true;
        return false;

    }
    public bool WallTest()
    {
        if (Raycast(Vector2.right, WallJumpableSurfaces))
        { 
            AudioSource.PlayClipAtPoint(this.Controller.PlayOnJump, AudioListener.FindObjectOfType<AudioListener>().transform.position);
            Controller.Velocity.x -= WallJumpForce.x;
            Controller.Velocity.y += WallJumpForce.y;
            return true;
        } else
        if (Raycast(-Vector2.right, WallJumpableSurfaces))
        { 
            AudioSource.PlayClipAtPoint(this.Controller.PlayOnJump, AudioListener.FindObjectOfType<AudioListener>().transform.position);
            Controller.Velocity.x += WallJumpForce.x;
            Controller.Velocity.y += WallJumpForce.y;
            return true;
        }
        return false;
    }
}
