﻿using UnityEngine;
using System.Collections;
using UnityEngine.Events;

public class Spawner : MonoBehaviour {
    public float InitialWait = 0;
    public GameObject Prefab;
    public float Cooldown = -1;
    public AudioClip PlayOnSpawn;
    public bool AttachToParent = false;
    public UnityEvent OnSpawn;
    public void Spawn()
    {
        if (PlayOnSpawn != null)
        {
            AudioSource.PlayClipAtPoint(PlayOnSpawn, this.transform.position);
        }
        var go = (GameObject)Instantiate(Prefab, this.transform.position, this.transform.rotation);
        if (AttachToParent)
        {
            go.transform.parent = this.transform.parent;
        }
        if (OnSpawn != null) OnSpawn.Invoke();
    }
    void OnEnable()
    {
        if (Cooldown > 0)
        {
            StartCoroutine(DoSpawn());
        }
    }
    void OnDisable()
    {
        StopAllCoroutines();
    }

    private IEnumerator DoSpawn()
    {
        if (InitialWait > 0)
        {
            yield return new WaitForSeconds(InitialWait);
        }
        while (true)
        {
            Spawn();
            yield return new WaitForSeconds(Cooldown);
        }
    }

}
