﻿using UnityEngine;
using System.Collections;
using UnityEngine.Events;

public class UseCooldown : MonoBehaviour {
    public float CooldownTime;
    public string AnimatorParameter = "On";
    public AudioClip OnSound, OffSound;
    public UnityEvent OnUse;
    public UnityEvent OnCooldownExpired;
    private Animator _animator;
    private Animator animator
    {
        get
        {
            if (_animator == null) _animator = this.GetComponent<Animator>();
            return _animator;
        }
    }

    public void Use()
    {
        if (inUse) return;

        StartCoroutine(DoUse());
    }
    void OnDisable()
    {
        StopAllCoroutines();
        NotInUse(true);
    }
    void OnEnable()
    {
        NotInUse(true);
    }
    private bool inUse = false;
    private IEnumerator DoUse()
    {
        InUse();

        yield return new WaitForSeconds(CooldownTime);

        NotInUse();

    }

    private void InUse(bool Silently = false)
    {
        inUse = true;
        animator.SetBool(AnimatorParameter, false);
        if (Silently) return;

        if (OnUse != null) OnUse.Invoke();
        AudioSource.PlayClipAtPoint(OnSound, this.transform.position);
    }

    private void NotInUse(bool Silently = false)
    {
        inUse = false;
        animator.SetBool(AnimatorParameter, true);
        if (Silently) return;

        if (OnCooldownExpired != null) OnCooldownExpired.Invoke();
        AudioSource.PlayClipAtPoint(OffSound, this.transform.position);
    }
}
